'use strict';

const fs = require('hexo-fs');
const qiniu = require('qiniu');

module.exports = class Deployer {
    constructor(options) {
        this.options = {
            mac: null,
            accessKey: null,
            secretKey: null,
            dirsToRefresh: [],
            tokenOptions: {
                scope: null
            },
            configOptions: {
                zone: null
            },
            cover: false,
            refreshCdn: false
        };
        Object.assign(this.options, options);

        this.options.mac = new qiniu.auth.digest.Mac(options.accessKey, options.secretKey);
        this.options.configOptions.zone = qiniu.zone[options.configOptions.zone];
    }

    createCdnManager(mac) {
        return new qiniu.cdn.CdnManager(mac);
    }

    createUploadToken(mac, putPolicyOptions) {
        return new qiniu.rs.PutPolicy(putPolicyOptions).uploadToken(mac);
    }

    createConfig(configOptions) {
        let config = new qiniu.conf.Config();
        return Object.assign(config, configOptions);
    }

    deploy(publicDir) {
        let that = this;
        return new Promise((resolve, reject) => {
            Promise.all(
                // traversal(publicDir, src => that.upload(src, pathlib.win32.relative(publicDir, src).replace(/\\/g, "/"))
                //     .finally(() => {
                //         console.log(src);
                //     })
                // )
                traversal(publicDir, src => {
                    console.log(src);
                    console.log(src.replace(/\\/g, "\/"));
                    console.log(pathlib.win32.relative(publicDir, src).replace(/\\/g, "/"));
                })
            ).then(() => {
                console.log("upload finish!");
                if (that.options.refreshCdn) {
                    that.createCdnManager(this.options.mac)
                        .refreshDirs(that.options.dirsToRefresh, (respErr, respBody, respInfo) => {
                            if (respInfo.statusCode === 200) {
                                resolve(respErr, respBody, respInfo);
                            } else {
                                reject(respErr, respBody, respInfo);
                            }
                        });
                }
            }).catch(() => {
                console.log("upload fail!");
            });
        }).then(() => {
            console.log("refresh finish!");
        }).catch(() => {
            console.log("refresh fail!");
        });
    }

    upload(src, key, tokenOptions, configOptions) {
        let uploadToken = this.createUploadToken(this.options.mac, Object.assign({}, this.options.tokenOptions, tokenOptions, this.options.cover ? {scope: this.options.tokenOptions.scope + ":" + key} : {}));
        let formUploader = new qiniu.form_up.FormUploader(this.createConfig(Object.assign({}, this.options.configOptions, configOptions)));
        let putExtra = new qiniu.form_up.PutExtra();
        return new Promise((resolve, reject) => {
            formUploader.putFile(uploadToken, key, src, putExtra, (respErr, respBody, respInfo) => {
                if (respInfo.statusCode === 200) {
                    resolve(respErr, respBody, respInfo);
                } else {
                    reject(respErr, respBody, respInfo);
                }
            });
        });
    }
};

module.exports = function (args) {
    if (!args.accessKey || !args.secretKey || !args.zone || !args.scope || !args.dirsToRefresh) {
        console.log(`You should configure deployment settings in _config.yml first!\n\n
        Example:\n
          deploy:\n
            type: qiniu\n
            accessKey: <accessKey>\n
            secretKey: <secretKey>\n
            zone: <zone>\n
            scope: <scope>\n
            expires: [expires]\n
            cover: [true|false]\n
        For more help, you can check the docs: ' http://hexo.io/docs/deployment.html'`);
        return;
    }
    const publicDir = this.public_dir;
    return new Deployer({
        accessKey: args.accessKey,
        secretKey: args.secretKey,
        dirsToRefresh: args.dirsToRefresh.split(","),
        tokenOptions: {
            scope: args.scope,
            expires: args.expires
        },
        configOptions: {
            zone: args.zone
        },
        cover: args.cover ? args.cover : true,
        refreshCdn: args.refreshCdn ? args.refreshCdn : true
    }).deploy(publicDir);
};
